
#include <SPI.h>
#include <Ethernet.h>
#include <SD.h>

// size of buffer used to capture HTTP requests
#define REQ_BUF_SZ   20

// MAC address from Ethernet shield sticker under board
byte mac[] = { 0x90, 0xA2, 0xDA, 0x0F, 0x67, 0xCD };
IPAddress ip(210,107,205,21); 

EthernetServer server(80);  // create a server at port 80
File webFile;
char HTTP_req[REQ_BUF_SZ] = {0}; // buffered HTTP request stored as null terminated string
char req_index = 0;              // index into HTTP_req buffer

byte Buf[64];

void setup()
{
    // disable Ethernet chip
    pinMode(10, OUTPUT);
    digitalWrite(10, HIGH);
    
   Serial.begin(9600);       // for debugging
   
    // initialize SD card
    Serial.println(F("Initializing SD card..."));
    if (!SD.begin(4)) {
        Serial.println(F("ERROR - SD card initialization failed!"));
        return;    // init failed
    }
    Serial.println(F("SUCCESS - SD card initialized."));
        
   // Ethernet.begin(mac);  // initialize Ethernet device  
    Ethernet.begin(mac,ip);  // initialize Ethernet device
    Serial.print("This IP address : ");
    IPAddress ip = Ethernet.localIP();
    Serial.println(ip);

    server.begin();           // start to listen for clients
}

void loop()
{
    int readCount = 0;
    float temperature, mVoltage;
   
    EthernetClient client = server.available();  // try to get client

    if (client) {  // got client?
        boolean currentLineIsBlank = true;
        while (client.connected()) {
            if (client.available()) {   // client data available to read
                char c = client.read(); // read 1 byte (character) from client
                // buffer first part of HTTP request in HTTP_req array (string)
                // leave last element in array as 0 to null terminate string (REQ_BUF_SZ - 1)
                if (req_index < (REQ_BUF_SZ - 1)) {
                    HTTP_req[req_index] = c;          // save HTTP request character
                    req_index++;
                }
                // print HTTP request character to serial monitor
                Serial.print(c);
                // last line of client request is blank and ends with \n
                // respond to client only after last line received
               if (c == '\n' && currentLineIsBlank) {
                    if (StrContains(HTTP_req, "GET / ") || StrContains(HTTP_req, "GET /index.htm")) {                
                          client.println(F("HTTP/1.1 200 OK"));
                          client.println(F("Content-Type: text/html"));
                          client.println(F("Connection: close"));  
                  	  client.println(F("Refresh: 3"));  
                          client.println();
                          client.println(F("<!DOCTYPE HTML><html><body><center>"));
                          client.println(F("<h1> Internet of Things </h1><br>"));
                          client.println(F("<img src=\"Ria.jpg\"/><br><h5>file size : 5KB</h5>"));             
                      
                          mVoltage =  ((float) analogRead(A0)) * 5000.0/1024.0;
                          temperature = ( mVoltage - 500.0 ) / 10.0;

                          client.print(F("Temperature : <font color = red>"));
                          client.print(temperature);
                          client.print(F("</font>  from <font color=blue>TMP36(temperature sensor)</font><br/><br/>"));
                      
                          if (analogRead(A1) > 300) {
                            client.print(F("Oh! It's <font color=red>daytime</font>. Let's work... study."));
                          } else { 
                            client.print(F("Oh! It is <font color=red>not bright!</font> "));
                           }  
                          client.print(F("<font color=blue> light sensor(Cds)! ^^")); 
                          client.println(F("<br/><br/>"));                
                          client.println(F("<img src=\"IoT.jpg\"/><br><h5>file size : 14KB</h5><br>")); 
			  client.println(F("<a href = http://cafe.naver.com/studyonarduino target=_blank><font size = 10 color = BLUE> Study On Arduino</font></a>"));
  		          client.println(F("<br>"));
                          client.println(F("<img src=\"Kp.jpg\"/><br><h5>file size : 88.4KB</h5>"));                    
                          client.println(F("</center></body></html>"));
                 } else if (StrContains(HTTP_req, "GET /IoT.jpg")) {
                        webFile = SD.open("IoT.jpg");        // open web page file
                         if (webFile) {
                            client.println(F("HTTP/1.1 200 OK"));
                            client.println();
                        }
                } else if (StrContains(HTTP_req, "GET /Ria.jpg")) {
                        webFile = SD.open("Ria.jpg");        // open web page file
                         if (webFile) {
                            client.println(F("HTTP/1.1 200 OK"));
                            client.println();
                        }                            
                 } else if (StrContains(HTTP_req, "GET /Ksp.jpg")) {
                        webFile = SD.open("Ksp.jpg");        // open web page file
                         if (webFile) { 
                            client.println(F("HTTP/1.1 200 OK"));
                            client.println();
                        } 
                 }      
           
                 if (webFile) {/*
                        while( webFile.available() ) {                          
                             readCount = webFile.read(Buf, 64);
                             client.write(Buf, readCount);
                        }*/
                        while( webFile.available() ) {                          
                             client.write(webFile.read());
                        }
                        
                        webFile.close();
                   }

                 req_index = 0;
                 StrClear(HTTP_req, REQ_BUF_SZ);
                 break;
               }
                // every line of text received from the client ends with \r\n
                if (c == '\n') {
                    // last character on line of received text
                    // starting new line with next character read
                    currentLineIsBlank = true;
                } 
                else if (c != '\r') {
                    // a text character was received from client
                    currentLineIsBlank = false;
                }
            } // end if (client.available())
        } // end while (client.connected())
        delay(1);      // give the web browser time to receive the data
        client.stop(); // close the connection
    } // end if (client)
}

// sets every element of str to 0 (clears array)
void StrClear(char *str, char length)
{
    for (int i = 0; i < length; i++) {
        str[i] = 0;
    }
}

// searches for the string sfind in the string str
// returns 1 if string found
// returns 0 if string not found
char StrContains(char *str, char *sfind)
{
    char found = 0;
    char index = 0;
    char len;

    len = strlen(str);
    
    if (strlen(sfind) > len) {
        return 0;
    }
    while (index < len) {
        if (str[index] == sfind[found]) {
            found++;
            if (strlen(sfind) == found) {
                return 1;
            }
        }
        else {
            found = 0;
        }
        index++;
    }

    return 0;
}
